/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.area.field;

import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.MultiLineString;
import com.vividsolutions.jts.geom.Point;
import java.beans.IntrospectionException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.area.AreaService;
import nl.cloudfarming.client.geoviewer.Geometrical;
import nl.cloudfarming.client.geoviewer.GeographicalAction;
import nl.cloudfarming.client.model.HelpLine;
import nl.cloudfarming.client.util.GeometryUtil;
import org.openide.util.Lookup;

/**
 * LayerObject representing a plot
 * 
 * @author Timon Veenstra
 */
public class HelpLineLayerObject implements Geometrical<MultiLineString, HelpLine> {

    private final MultiLineString geometry;
    private final HelpLine helpLine;
    private AreaService service;
    private static final Logger LOG = Logger.getLogger("LineLayerObject.Actions");

    public HelpLineLayerObject(final HelpLine helpLine) throws IntrospectionException {
        this.helpLine = helpLine;
        geometry = (MultiLineString) GeometryUtil.wktToGeometry(helpLine.getWKT());
        service = getPlotAreaService();
    }

    /**
     * @see nl.cloudfarming.client.geoviewer.LayerObject#getGeometry() 
     * @return The Geometry of type MultiLineString
     */
    @Override
    public MultiLineString getGeometry() {
        return this.geometry;
    }

    /**
     * @see nl.cloudfarming.client.geoviewer.LayerObject#getObject() 
     * @return The Object of type HelpLine this LayerObject is representing 
     */
    @Override
    public HelpLine getObject() {
        return this.helpLine;
    }

    /**
     * @see nl.cloudfarming.client.geoviewer.LayerObject#getKey() 
     * @return Key as String
     */
    @Override
    public String getKey() {
        return Long.toString(this.helpLine.getId());
    }

    @Override
    public Long getKeyAttributeValue() {
        return this.helpLine.getId();
    }

    /**
     * @see nl.cloudfarming.client.geoviewer.LayerObject#getTooltipText() 
     * @return TooltipText as String
     */
    @Override
    public String getTooltipText() {
        return Long.toString(this.helpLine.getId()) + " " + this.helpLine.getName();
    }

    /**
     * @see nl.cloudfarming.client.geoviewer.LayerObject#getId() 
     * @return The Id of the wrapped object
     */
    @Override
    public long getId() {
        return this.helpLine.getId();
    }

    /**
     * @see nl.cloudfarming.client.geoviewer.LayerObject#getActions() 
     * @return Array with actions for this LayerObject
     */
    @Override
    public GeographicalAction[] getActions() {
        List<GeographicalAction> layerObjectActions = new ArrayList<GeographicalAction>();
        for (LineAction lineAction : Lookup.getDefault().lookupAll(LineAction.class)) {
            try {
                layerObjectActions.add(lineAction.getClass().newInstance());
            } catch (InstantiationException ex) {
                LOG.log(Level.FINE, "Instantiation of new {0} failed", lineAction.getClass());
            } catch (IllegalAccessException ex) {
                LOG.log(Level.FINE, "Instantiation of new {0} failed", lineAction.getClass());
            }
        }
        return layerObjectActions.toArray(new GeographicalAction[]{});
    }

    /**
     * Returns the service this class is using. Primarily used for unittesting.
     * @return (PlotArea) Service this class is using
     */
    protected final AreaService getPlotAreaService() {
        return (service == null) ? Lookup.getDefault().lookup(AreaService.class) : service;
    }

    /**
     * Returns the service this class is using. Primarily used for unittesting.
     * @param service (PlotArea) Service this class should be using
     */
    protected void setPlotAreaService(final AreaService service) {
        this.service = service;
    }

    @Override
    public MultiLineString getRenderObject(Envelope envelope) {
        if (envelope.intersects(geometry.getEnvelopeInternal())){
            return geometry;
        }
        return null;
    }

    @Override
    public Point getCentroid() {
        return this.geometry.getCentroid();
    }
        
}
